%
%   NANAHX.M  Simple Model of Sodium-Sodium Shell and Tube Heat Exchanger 
%  
%   -->  assumes incompressible fluids on both shell & tube side
%   -->  also assumes constant properties and heat transfer coefficients
%   -->  model can address variable number of nodes along axial direction (user input)
%   -->  can use either parallel or counter flow nodal model
%   -->  can input heat transfer coeff or compute internally (both axial and cross 
%          flow correlations are available on shell side)
%
%   See the System Dynamics Course Notes Section VI Case D for a detailed development
%   of this model...
%
%   also uses two script files for printing data to output file:
%      NANAHXP1.M - edits basic material data and heat exchanger info
%      NANAHXP2.M - edits data for internal computation of heat transfer coeffs
%                   and final HT coeffs used
%
%   file prepared by J. R. White, UMass-Lowell (last update: Feb. 2020)
%

      clear all,   close all,   nfig = 0;
%
%   steel properties (converted to proper units)
%   (1 % carbon steel at 750 F from J. P. Holman p. 535 Table A-2)
      cpst = 0.473*0.23884;    % specific heat (Btu/lbm-F)
      rhost = 7800/16.0185;    % density (lbm/ft^3)
      kst = 36/1.73028;        % thermal conductivity (Btu/hr-ft-F)
%
%   sodium properties
%   (at 750 F from El Wakil p. 268 Table 10-2)
      cp = 0.306;        % specific heat (Btu/lbm-F)
      rho = 53.63;       % density (lbm/ft^3)
      k = 42.15;         % thermal conductivity (Btu/hr-ft-F)
      mu = 0.687;        % viscosity (lbm/ft-hr)
      Pr = cp*mu/k;      % prandtl number (dimensionless)
%
%   data for heat exchanger 
%   (mostly from Annals of Nucl Eng paper by Kutbi (1991))
%
%   physical data
      Tod = 0.625/12;   % tube outside dia (ft) (from Table 2)
      Tid = 0.527/12;   % tube inside dia (ft)  (from Table 2)
      Att = 4900;       % total tube surface area (ft^2)  (from Table 2)
%
%   NOTE: The number of tubes and shell inside diameter were not given in the 
%   paper by Kutbi.  The choice of these parameters has an affect on the
%   the computed flow rates and derived heat transfer coefficients.  The values 
%   needed to give reasonable comparison to the Kutbi HT coeffs are 3000 tubes
%   and 44 in, respectively (for cross flow correlation).  However, this 
%   value of shell diameter seems too small for a reasonable mass of sodium on
%   the shell side.  Therefore, let's try Ntube = 2000 and Sid = 60 in. since 
%   these give reasonable comparison to the steady state data given in Kutbi 
%   (ie. the outlet temperatures match within a few degrees on both the shell 
%   and tube sides for the 100 node case).
%
%   *** remaining data are assumed ***
      Ntube = 2000;         % number of tubes       
      Sid = 60/12;          % shell inside dia (ft)
      Npass = 1;            % number of tube passes
%
%   steady state operating data (from Table 2 in Kutbi)
      Tsin = 883;           % shell side inlet temp (F)
      Tsotss = 700;         % shell side outlet temp (F)
      mdsss = 3.81e+6;      % shell side mass flow rate (lbm/hr)
      Ttin = 586;           % tube side inlet temp (F)
      Ttotss = 872;         % tube side outlet temp (F)
      mdtss = 2.44e+6;      % tube side mass flow rate (lbm/hr) (modified slightly)
      Qss = mdsss*cp*(Tsin-Tsotss);   % total heat transferred
%
%   computed parameters
      L = Att/(pi*Tod*Ntube*Npass);   % tube length 
      Vtot = (pi*Sid*Sid/4)*L;        % total vol inside shell
      Ltot = L*Ntube*Npass;           % total length of all tubes
      Vtube = (pi*Tod*Tod/4)*Ltot;    % vol occupied by tubes + tube fluid
      Vs = Vtot-Vtube;                % shell side volume
      Vt = (pi*Tid*Tid/4)*Ltot;       % tube side volume
      Vm = Vtube-Vt;                  % tube metal volume
      Phs = pi*Tod*Ntube*Npass;       % heated perimeter on shell side
      As = pi*Tod*Ltot;               % shell side heat transfer area
      At = pi*Tid*Ltot;               % tube side heat transfer area
      Am = (pi/4)*(Tod^2-Tid^2)*Ntube*Npass;  % conduction heat transfer area in metal
%
%   write summary of basic material and geometry data to output file
      fout = fopen('nanahx.out','wt');
      nanahxp1
%
%   now determine heat transfer coeff (2 options)
      htopt = menu('Select Heat Transfer Calc Option', ...
                   '        Use Data from Kutbi        ', ...
                   '     Use Standard Correlations     ');                
      if htopt == 1   
        hs = 6750;   % steam side HT coeff from Kutbi Table 3 (BTU/hr-ft^2-F)
        ht = 4650;   % tube side HT coeff from Kutbi Table 3 (BTU/hr-ft^2-F)
        titlht = 'Kutbi HT Data';
      end
      if htopt == 2  
        titlht = 'Std. HT Correlations';
        flopt = menu('Select Flow Option for HT Coeff Calc ', ...
                     '       Use Cross Flow Correlation       ', ...
                     '       Use Axial Flow Correlations      '); 
        mult = 1;   titlfl = 'Axial Flow Correl.';  
        if flopt == 1,   mult = 2;   titlfl = 'Cross Flow Correl.';   end               
%
%   calc heat transfer coeff on inside of tube
        Aft = (pi*Tid*Tid/4)*Ntube;   % tube flow area
        Velt = mdtss/(rho*Aft);       % flow velocity tube side
        Ret = rho*Velt*Tid/mu;        % Reynolds number tube side
        Pet = Ret*Pr;                 % Peclet number tube side
        Nut = 0.625*Pet^(0.4);        % Nusselt number tube side
        ht = k*Nut/Tid;               % heat transfer coeff tube side
%
%   calc heat transfer coeff on shell side
        Afs = Vs/L;                   % shell side flow area
        De = 4*Afs/Phs;               % equiv diameter
        Vels = mdsss/(rho*Afs);       % flow velocity shell side
        Res = rho*Vels*De/mu;         % Reynolds number shell side
        Pes = Res*Pr;                 % Peclet number shell side
        Nus = mult*0.106*Pes^(0.6);   % Nusselt number shell side
        hs = k*Nus/De;                % heat transfer coeff shell side
      end
%
%   write heat transfer data to output file
      nanahxp2
%
%   close output file
      fprintf(fout,'\n\n');
      fprintf(fout,'  Steady state & dynamic analysis results are summarized in the plots... \n');
      fclose(fout);
%
%   now let's set up the state space matrices (assume constant mass flow rates)
%
%   define number of nodes and direction of flow
      N = input('Input number of nodes in model (N): ');
      flowdir = menu('Select tube side flow direction', ...
                     '          PARALLEL FLOW          ', ...
                     '          COUNTER FLOW           ');                
      if flowdir == 1,  titldir = 'Parallel Flow Case';   end
      if flowdir == 2,  titldir = 'Counter Flow Case';    end
%
%   setup space for matrices (2 inputs - mass flow rates are treated as constants)
      A = zeros(3*N,3*N);   B = zeros(3*N,2);   C = zeros(2,3*N);   D = zeros(2,2);
%
%   calculate capacitances and resistances
      Cs = rho*cp*Vs/N;     Cm = rhost*cpst*Vm/N;     Ct = rho*cp*Vt/N;
      Rfs = 1/(mdsss*cp);   Rft = 1/(mdtss*cp);
      Rms = N/(hs*As);      Rmt = N/(ht*At);          Rmm = Ltot/(kst*Am);
%
%   setup shell side equations
      A(1,1) = -(1/Rfs + 1/Rms)/Cs;   A(1,N+1) = (1/Rms)/Cs;   B(1,1) = (1/Rfs)/Cs;
      for i = 2:N
        A(i,i-1) = (1/Rfs)/Cs;     A(i,N+i) = (1/Rms)/Cs;
        A(i,i) = -(1/Rfs + 1/Rms)/Cs;
      end
%
%   setup tube metal equations
      for i = 1:N
        ii = N+i;  iii = 2*N+i;
        A(ii,i) = (1/Rms)/Cm;     A(ii,iii) = (1/Rmt)/Cm;
        A(ii,ii) = -(1/Rms + 1/Rmt + 2/Rmm)/Cm;  
        if i == 1, A(ii,ii) = A(ii,ii)+(1/(Rmm*Cm));  end  % remove 1 metal neighbor
        if i == N, A(ii,ii) = A(ii,ii)+(1/(Rmm*Cm));  end  % remove 1 metal neighbor
      end
      A(N+1,N+2) = 1/(Rmm*Cm);       A(2*N,2*N-1) = 1/(Rmm*Cm); 
      if N > 2
        for i = 2:N-1
          ii = N+i;   A(ii,ii-1) = 1/(Rmm*Cm);   A(ii,ii+1) = 1/(Rmm*Cm);
        end
      end
%
%   setup tube side equations (slight difference depending on flow direction)
%   for parallel flow (flow is from node 2N+1 to 3N)
      if flowdir == 1
        ii = 2*N+1;   iii = N+1;
        A(ii,ii) = -(1/Rft + 1/Rmt)/Ct;  A(ii,iii) = (1/Rmt)/Ct;  B(ii,2) = (1/Rft)/Ct;
        for i = 2:N
          ii = 2*N+i;   iii = N+i;
          A(ii,ii-1) = (1/Rft)/Ct;      A(ii,iii) = (1/Rmt)/Ct;
          A(ii,ii) = -(1/Rft + 1/Rmt)/Ct; 
       end
      end
%   for counter flow (flow is from node 3N to 2N+1)
      if flowdir == 2
        ii = 3*N;   iii = 2*N;
        A(ii,ii) = -(1/Rft + 1/Rmt)/Ct;
        A(ii,iii) = (1/Rmt)/Ct;  
        B(ii,2) = (1/Rft)/Ct;
        for i = 2:N
          ii = 3*N-(i-1);   iii = 2*N-(i-1);
          A(ii,ii+1) = (1/Rft)/Ct;      A(ii,iii) = (1/Rmt)/Ct;
          A(ii,ii) = -(1/Rft + 1/Rmt)/Ct; 
       end
      end
%
%   set output as the outlet temps for the two fluid streams
      C(1,N) = 1;   
      if flowdir == 1,   C(2,3*N) = 1;   end
      if flowdir == 2,   C(2,2*N+1) = 1;   end
%
%   edit system matrices if N is small (some debug edit...)
      if N < 4,   disp('System Matrices'), A, B, C, D,    end
%
%   solve for steady state profiles (this is an important test of the model)
      uss = [Tsin; Ttin];      xss = A\(-B*uss);   
      exss = [xss(1:N) xss(N+1:2*N) xss(2*N+1:3*N)];
      disp('Steady State Solution based on System Model')
      disp('   Tshell    Tmetal     Ttube'),   disp(exss)
      nnum1 = 0:N;  nnum = 0.5:N-0.5;   
      Tsss = [Tsin; xss(1:N)];   Tmss = xss(N+1:2*N);
      if flowdir == 1,  Ttss = [Ttin; xss(2*N+1:3*N)];  end
      if flowdir == 2,  Ttss = [xss(2*N+1:3*N); Ttin];  end
      nfig = nfig+1;   figure(nfig)
      plot(nnum1,Tsss,'r',nnum,Tmss,'g:',nnum1,Ttss,'b--','LineWidth',2),grid
      title(['Steady State Temperature Distribution  (',num2str(N),' nodes)'])
      xlabel('Node Number'),ylabel('Temperature (F)')
      v = axis;   xt = v(1)+0.10*(v(2)-v(1));
      yt1 = v(3) + 0.15*(v(4)-v(3)); 
      yt2 = v(3) + 0.10*(v(4)-v(3));
      yt3 = v(3) + 0.05*(v(4)-v(3));
      text(xt,yt1,titldir),text(xt,yt2,titlht)
      if htopt == 2,  text(xt,yt3,titlfl),   end
      legend('Ts','Tm','Tt')
%
%   find eigenvalues  (more debug edit...)
      if N < 4,  disp('Eigenvalues for system matrix'),   eig(A)',  end
%
%   do dynamic analyis for N < 51  (10 deg step change in inputs)
      if N <= 50
      sys = ss(A,10*B,C,D);
      too = 0;   tff = 0.6/60;   t = linspace(too,tff,61);     y = step(sys,t);
%
%   plot step response for 10 deg change in inlet shell side temp
      nfig = nfig+1;   figure(nfig)
      subplot(2,1,1),plot(60*t,y(:,1,1),'r',60*t,y(:,2,1),'b--','LineWidth',2),grid
      title(['Change in Outlet Temps for 10 deg Step in Tsin  (', ...
             num2str(N),' nodes)'])
      ylabel('Temp Change (F)')
      v = axis;   v(2) = 60*tff;   axis(v);
      xt = v(1)+0.70*(v(2)-v(1)); 
      yt1 = v(3) + 0.70*(v(4)-v(3)); 
      yt2 = v(3) + 0.60*(v(4)-v(3));
      yt3 = v(3) + 0.50*(v(4)-v(3));
      text(xt,yt1,titldir),text(xt,yt2,titlht)
      if htopt == 2,  text(xt,yt3,titlfl),   end
      legend('Tsout','Ttout')
%
%   plot step response for 10 deg change in inlet tube side temp
      subplot(2,1,2),plot(60*t,y(:,1,2),'r',60*t,y(:,2,2),'b--','LineWidth',2),grid
      title(['Change in Outlet Temps for 10 deg Step in Ttin  (', ...
             num2str(N),' nodes)'])
      xlabel('Time (min)'),ylabel('Temp Change (F)')
      v = axis;   v(2) = 60*tff;   axis(v);
      xt = v(1)+0.70*(v(2)-v(1)); 
      yt1 = v(3) + 0.70*(v(4)-v(3)); 
      yt2 = v(3) + 0.60*(v(4)-v(3));
      yt3 = v(3) + 0.50*(v(4)-v(3));
      text(xt,yt1,titldir),text(xt,yt2,titlht)
      if htopt == 2,  text(xt,yt3,titlfl),   end
      legend('Tsout','Ttout')
%
      end
%
%   end of simulation
